<?php

namespace Drupal\Tests\genpass\Functional;

use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\User;

/**
 * Tests tokens added by Generate Password.
 *
 * @group genpass
 */
class PasswordTokenTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'user',
    'genpass',
    'language',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A basic user with "view content" permission.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $webUser;

  /**
   * Password generator service.
   *
   * @var \Drupal\Core\Password\PasswordGeneratorInterface
   */
  protected $passwordGenerator;

  /**
   * {@inheritdoc}
   */
  protected function setUp():void {
    parent::setUp();

    // Required services.
    $this->passwordGenerator = $this->container->get('password_generator');

    // Create normal user.
    $permissions = [
      'access content',
    ];
    $this->webUser = $this->drupalCreateUser($permissions);

    // Log in as admin.
    $this->drupalLogin($this->rootUser);
  }

  /**
   * Test user tokens added by Generate Password.
   */
  public function testUserPasswordTokens() {

    $token_service = \Drupal::token();
    $language_interface = \Drupal::languageManager()->getCurrentLanguage();

    // Password set on user creation has already been hashed. Set password to
    // known new generated password.
    $password = $this->passwordGenerator->generate();
    $this->webUser->setPassword($password);

    // Mimic how the user entity is setup just before sending out new account
    // emails. Must use Markup to allow the property to be set without safe
    // string escaping.
    $this->webUser->password = Markup::create($password);

    // Build list of tokens and their expected contents for the webUser.
    $tests = [
      '[user:password]' => $this->webUser->getPassword(),
      '[account:password]' => $this->webUser->getPassword(),
    ];

    // Replace the tokens in each input and compare to expected result.
    foreach ($tests as $input => $expected) {
      $bubbleable_metadata = new BubbleableMetadata();
      $output = $token_service->replace(
        $input,
        ['user' => $this->webUser],
        ['langcode' => $language_interface->getId()],
        $bubbleable_metadata
      );
      $this->assertSame((string) $expected, (string) $output, "Failed test case: {$input}");
    }

    // Build list for anonymous user. They don't have a password to set.
    $anonymous_user = User::load(0);
    $tests = [
      '[user:password]' => 'Your password',
      '[account:password]' => 'Your password',
    ];

    // Replace the tokens in each input and compare to expected result.
    foreach ($tests as $input => $expected) {
      $bubbleable_metadata = new BubbleableMetadata();
      $output = $token_service->replace(
        $input,
        ['user' => $anonymous_user],
        ['langcode' => $language_interface->getId()],
        $bubbleable_metadata
      );
      $this->assertSame((string) $expected, (string) $output, "Failed test case: {$input}");
    }
  }

}
