<?php

namespace Drupal\Tests\genpass\FunctionalJavascript;

use Drupal\Tests\BrowserTestBase;
use Drupal\genpass\GenpassInterface;

/**
 * Tests functionality of the Generate Password module.
 *
 * @group genpass
 */
class GenpassTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = TRUE;

  /**
   * Modules to install.
   *
   * @var array
   */
  protected static $modules = [
    'user',
    'genpass',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with "administer account settings" .
   *
   * And "administer users" permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $webUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp():void {
    parent::setUp();

    $permissions = [
      'view the administration theme',
      'administer account settings',
      'administer users',
    ];

    $this->webUser = $this->drupalCreateUser($permissions);
    $this->drupalLogin($this->webUser);
  }

  /**
   * Test Generate Password configs and create users by admin.
   */
  public function testGenpassConfigsAndCreateUsersByAdmin() {

    // Configure Account settings with Generate Password options.
    $this->drupalGet('admin/config/people/accounts');
    $this->assertSession()->pageTextContains('Account settings');
    $this->assertSession()->pageTextContains('Generate Password - User Account Registration');
    $this->assertSession()->pageTextContains('User password entry');
    $this->assertSession()->pageTextContains('Admin password entry');
    $this->assertSession()->pageTextContains('Generated password length');
    $this->assertSession()->pageTextContains('Generated password display');

    $this->getSession()->getPage()->selectFieldOption('genpass_mode', '2');
    $this->getSession()->getPage()->pressButton('Save configuration');

    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create the test_authenticated user.
    $this->drupalGet('admin/people/create');
    $this->assertSession()->pageTextContains('Add user');
    $this->getSession()->getPage()->fillField('mail', 'authenticated.test@drupal.org');
    $this->getSession()->getPage()->fillField('Username', 'test_authenticated');
    $this->getSession()->getPage()->pressButton('Create new account');
    $this->assertSession()->pageTextContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated. No email has been sent.');
  }

  /**
   * Test Generate Password hide password field functionality.
   */
  public function testGenpassHidePasswordField() {

    // Allow admins to set passwords.
    $this->drupalGet('admin/config/people/accounts');
    $this->assertSession()->pageTextContains('Admin password entry');

    // Allow admins to set password.
    $this->getSession()->getPage()->selectFieldOption(
      'genpass_admin_mode',
      (string) GenpassInterface::PASSWORD_ADMIN_SHOW
    );
    $this->getSession()->getPage()->pressButton('Save configuration');

    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create the test_authenticated user.
    $this->drupalGet('admin/people/create');
    $this->assertSession()->pageTextContains('Provide a password for the new account in both fields.');
    $this->getSession()->getPage()->fillField('mail', 'authenticated.test@drupal.org');
    $this->getSession()->getPage()->fillField('Username', 'test_authenticated');
    $this->getSession()->getPage()->pressButton('Create new account');
    $this->assertSession()->pageTextContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated. No email has been sent.');

    // Disallow admins to set passwords and not display passwords (default).
    $this->drupalGet('admin/config/people/accounts');
    $this->getSession()->getPage()->selectFieldOption(
      'genpass_admin_mode',
      (string) GenpassInterface::PASSWORD_ADMIN_HIDE
    );
    $this->getSession()->getPage()->selectFieldOption(
      'genpass_display',
      (string) GenpassInterface::PASSWORD_DISPLAY_NONE
    );
    $this->getSession()->getPage()->pressButton('Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create the test_authenticated_two user. As ADMIN_HIDE and DISPLAY_NONE
    // are set, it will not contain the text "did not provide a password".
    $this->drupalGet('admin/people/create');
    $this->assertSession()->pageTextNotContains('Provide a password for the new account in both fields.');
    $this->getSession()->getPage()->fillField('mail', 'authenticated.test2@drupal.org');
    $this->getSession()->getPage()->fillField('Username', 'test_authenticated_two');
    $this->getSession()->getPage()->pressButton('Create new account');
    $this->assertSession()->pageTextNotContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextNotContains('The password is:');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated_two. No email has been sent.');

    // Show passwords to admin.
    $this->drupalGet('admin/config/people/accounts');
    $this->getSession()->getPage()->selectFieldOption(
      'genpass_display',
      (string) GenpassInterface::PASSWORD_DISPLAY_ADMIN
    );
    $this->getSession()->getPage()->pressButton('Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Repeat, but display the password.
    $this->drupalGet('admin/people/create');
    $this->getSession()->getPage()->fillField('mail', 'authenticated.test3@drupal.org');
    $this->getSession()->getPage()->fillField('Username', 'test_authenticated_three');
    $this->getSession()->getPage()->pressButton('Create new account');
    $this->assertSession()->pageTextNotContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextContains('The password is:');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated_three. No email has been sent.');
  }

}
